﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace Lindenmayer
{
    public partial class Form1 : Form
    {

        /*********************************************
         * Champs de classe
         */ 

        /// <summary>
        /// Point de départ
        /// </summary>
        private PointF start=new PointF(100,100);

        /// <summary>
        /// Longueur d'un segment
        /// </summary>
        private float length = 50;

        /// <summary>
        /// Angle de rotation
        /// </summary>
        private float angle=(float)(Math.PI/3.0);

        /// <summary>
        /// Direction initiale
        /// </summary>
        private float angleStart = 0;

        /// <summary>
        /// Profondeur de récursivité
        /// </summary>
        private int profondeur=0;

        /// <summary>
        /// Séquence initiale
        /// </summary>
        private string sequence = "F+F+F+F+F+F+";

        /// <summary>
        /// Liste des règles de transfromation
        /// </summary>
        private List<Regle> regles = new List<Regle>();

       /*********************************************
       * Gestion des évènements
       */ 

        /// <summary>
        /// Constructeur
        /// </summary>
        public Form1()
        {
            InitializeComponent();
        }

        /// <summary>
        /// SLider pour la valeur de profondeur
        /// </summary>
        /// <param name="sender">Voir MSDN</param>
        /// <param name="e">voir MSDN</param>
        private void trackBar1_Scroll(object sender, EventArgs e)
        {
            label1.Text = "Profondeur : " + trackBar1.Value;
        }

        /// <summary>
        /// CLic event sur le bouton générer
        /// </summary>
        /// <param name="sender">Voir MSDN</param>
        /// <param name="e">Voir MSDN</param>
        private void button1_Click(object sender, EventArgs e)
        {
            float startx;
            float starty;
            float longueur;
            float angle;
            float angleStart;
            int prof;
            // récupération de la valeur des textbox et conversion en valeur numérique
            if (!Single.TryParse(textBox2.Text, out startx))
            {
                MessageBox.Show("Valeur 'start x' incorrecte !");
                textBox2.SelectAll();
                textBox2.Focus();
                return;
            }

            if (!Single.TryParse(textBox3.Text, out starty))
            {
                MessageBox.Show("Valeur 'start y' incorrecte !");
                textBox3.SelectAll();
                textBox3.Focus();
                return;
            }

            if (!Single.TryParse(textBox4.Text, out longueur))
            {
                MessageBox.Show("Valeur 'longueur' incorrecte !");
                textBox4.SelectAll();
                textBox4.Focus();
                return;
            }

            if (!Single.TryParse(textBox5.Text, out angle))
            {
                MessageBox.Show("Valeur 'angle' incorrecte !");
                textBox5.SelectAll();
                textBox5.Focus();
                return;
            }

            if (!Single.TryParse(textBox6.Text, out angleStart))
            {
                MessageBox.Show("Valeur 'angle start' incorrecte !");
                textBox6.SelectAll();
                textBox6.Focus();
                return;
            }

            prof = trackBar1.Value;


            // affectation des valeurs aux champs
            this.start = new PointF(startx, starty);
            this.length = longueur;
            this.angle = (float)(Math.PI * angle / 180.0); // conversion degré -> radian
            this.angleStart = (float)(Math.PI * angleStart / 180.0); // conversion degré -> radian
            this.profondeur = prof;

            // lecture des règles de conversion
            regles.Clear();
            for (int i = 0; i < dataGridView1.RowCount; i++)
            {
                object o = dataGridView1.Rows[i].Cells[0].Value;
                if (o != null && o is string)
                {
                    string st = (string)o;
                    if (st.Length > 0)
                    {
                        char c = st[0];
                        o = dataGridView1.Rows[i].Cells[1].Value;
                        if (o != null && o is string)
                        {
                            regles.Add(new Regle(c, (string)o));
                        }
                    }
                }


            }


            // on applique les règles de transformation
            this.sequence = AppliquerReglesDeSubstitution(new StringBuilder(textBox1.Text), profondeur).ToString();
            
            // Pour le débugage
            Console.WriteLine(sequence);

            // On rafraichit l'affichage
            panel1.Invalidate();
        }

        /// <summary>
        /// Paint event sur la zone de dessin
        /// </summary>
        /// <param name="sender">Voir MSDN</param>
        /// <param name="e">voir MSDN</param>
        private void panel1_Paint(object sender, PaintEventArgs e)
        {
            BufferedGraphics bg = BufferedGraphicsManager.Current.Allocate(e.Graphics, e.ClipRectangle);
            Graphics g = bg.Graphics;
            g.Clear(Color.White);

            // appelle du traceur
            trace(g, sequence, start, angleStart, length);
            
            bg.Render();
            bg.Dispose();
        }


       /*********************************************
       * Système de Lindemayer
       */ 

        /// <summary>
        /// Un stylo pour dessiner
        /// </summary>
        private Pen pen=new Pen(Color.Black);


        /// <summary>
        /// Calcule la nouvelle position après avoir avancé dans la direction angle en partant du point start.
        /// </summary>
        /// <param name="start">Point de départ</param>
        /// <param name="angle">Direction</param>
        /// <param name="length">Distance à parcourir</param>
        /// <returns>Point d'arrivé</returns>
        private PointF Avance(PointF start, float angle, float length)
        {
            return new PointF(start.X + length * (float)Math.Cos(angle), start.Y + length * (float)Math.Sin(angle));
        }

        /// <summary>
        /// Dessine la séquence 
        /// </summary>
        /// <param name="g">Graphics pour le dessin</param>
        /// <param name="sequence">Chaine de caractère codant le dessin à réaliser</param>
        /// <param name="start">Position de départ du traceur</param>
        /// <param name="curAngle">Direction de départ</param>
        /// <param name="length">Longueur du segment de base</param>
        private void trace(Graphics g, String sequence, PointF start, float curAngle, float length)
        {
            
            // A compléter
          
        }

        /// <summary>
        /// Cherche la règle de substitution associée à un caractère
        /// </summary>
        /// <param name="regles">Liste de règles</param>
        /// <param name="c">Caractère à substituer</param>
        /// <returns>La première règle correspondant au caractère trouvée dans la liste ou null si aucune règle ne correspond</returns>
        private Regle TrouverRegle(List<Regle> regles, char c)
        {

            // A compléter

            return null;
        }


        /// <summary>
        /// Applique récursivement les règles de substitutions à une séquence
        /// </summary>
        /// <param name="sequence">La séquence à traiter</param>
        /// <param name="profondeur">Nombre de récursion à réaliser</param>
        /// <returns>La séquence avec les règles de substitution appliquées n-fois avec n=profondeur</returns>
        private StringBuilder AppliquerReglesDeSubstitution(StringBuilder sequence, int profondeur)
        {
           
            // A compléter

            return sequence;
        }




    }

       
}
